import os
from argparse import ArgumentParser, Namespace
import logging
import datetime

from playvision.storage.swift import SwiftSelectelStorage
from playvision.database.couchbase import CouchbaseBackup

COUCH_PASS = os.environ['COUCH_PASS']
SWIFT_PASS = os.environ['SWIFT_PASS']

logging.basicConfig(level=logging.INFO)


def parse_args() -> Namespace:
    parser = ArgumentParser()
    parser.add_argument('--couch-user', required=True, type=str, help='Couchbase user')
    parser.add_argument('--couch-url', required=True, type=str, help='Couchbase URL')
    parser.add_argument('--swift-user', required=True, type=str, help='Selectel Cloud Storage user')
    parser.add_argument('--sel-id', required=True, type=str, help='Selectel accound id: 12345')
    parser.add_argument('--sel-project-id', required=True, type=str, help='Selectel project id')
    parser.add_argument('--bucket-name', required=True, type=str, help='Swift bucket name')
    parser.add_argument('--backup-name', required=True, type=str, help='Backup name prefix')
    return parser.parse_args()


def main():
    args = parse_args()
    cb = CouchbaseBackup(args.couch_user, COUCH_PASS, args.couch_url)
    backupfile = cb.make_backup_archive()
    today = datetime.date.today().isoformat()
    storage = SwiftSelectelStorage(args.swift_user, SWIFT_PASS, args.sel_id, args.sel_project_id)
    ttl_15_days = 15*24*60*60
    split_size = 512*1024*1024 # 512mb
    storage.upload(args.bucket_name, backupfile, f'{today}-{args.backup_name}', split_size, ttl_15_days)
    os.unlink(backupfile)



import subprocess
import sys
import logging


class SwiftSelectelStorage:
    def __init__(self, swift_user: str, swift_password: str, sel_id: str, sel_project_id: str):
        self.region_name = 'ru-1'
        self.auth_url = 'https://api.selcdn.ru'
        self.auth_version = '2'
        self.storage_url = f'https://api.selcdn.ru/v1/SEL_{sel_id}'
        self.username = swift_user
        self.password = swift_password
        self.project_id = sel_project_id
        self.tenant_id = sel_project_id
        self.general_args = [
            'swift',
            '--auth-version', self.auth_version,
            '--os-auth-url', self.auth_url,
            '--os-region-name', self.region_name,
            '--os-storage-url', self.storage_url,
            '--os-username', self.username,
            '--os-password', self.password,
            '--os-project-id', self.project_id,
            '--os-tenant-id', self.project_id
        ]

    def upload(self, bucket: str, src_file: str, dest_file: str, split_size_bytes: int = None, ttl_sec: int = None):
        add_args_split = []
        add_args_ttl = []
        if split_size_bytes:
            add_args_split = [
                '-S', str(split_size_bytes),
                '--segment-container', f'{bucket}-segments',
                '--use-slo',
            ]
        if ttl_sec:
            add_args_ttl = ['--header', f'X-Delete-After:{ttl_sec}' ]
        cmd = self.general_args + [
            'upload',
            '--object-name', dest_file
        ]
        cmd = cmd + add_args_split + add_args_ttl
        cmd += [bucket, src_file]
        try:
            result = subprocess.run(cmd, check=True)
        except subprocess.CalledProcessError as e:
            logging.error('Cant upload to bucket')
            logging.error(e)
            sys.exit(1)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="playvision-devops-tools",
    version="0.0.9",
    author="Dmitriy Shelestovskiy",
    author_email="one@sonhador.ru",
    description="Playvision devops tools",
    long_description=long_description,
    long_description_content_type="text/markdown",
    # url="https://github.com/pypa/sampleproject",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3"
    ],
    python_requires='>=3.6',
    entry_points={
        'console_scripts': [
            'backuper=playvision.cli:backup'
        ]
    },
    install_requires=[
        "python-keystoneclient==4.2.0",
        "python-swiftclient==3.12.0"
    ]
)



import os, sys
import tempfile
import subprocess
import tarfile
import shutil
import logging


class CouchbaseBackup:
    def __init__(self, couch_user, couch_pass, couch_url):
        self.args = [
            'cbbackup',
            '-u', couch_user,
            '-p', couch_pass,
            couch_url
        ]
        self.tempdir = tempfile.mkdtemp()

    def make_backup_archive(self) -> str:
        self._backup()
        fd, tar_fname = tempfile.mkstemp(suffix='.tgz')
        os.close(fd)
        logging.info(f'Compress {self.tempdir} to {tar_fname}')
        with tarfile.open(tar_fname, 'w:gz') as tar:
            tar.add(self.tempdir)
        return tar_fname

    def _backup(self):
        self.args.append(self.tempdir)
        try:
            result = subprocess.run(self.args, check=True, capture_output=True)
            logging.info(result)
        except subprocess.CalledProcessError as e:
            logging.error('cant create backup')
            logging.error(e)
            sys.exit(1)

    def __del__(self):
        shutil.rmtree(self.tempdir)
